"use strict";

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _reactDom = require("react-dom");

var _reactDom2 = _interopRequireDefault(_reactDom);

var _reactPointable = require("react-pointable");

var _reactPointable2 = _interopRequireDefault(_reactPointable);

var _fp = require("lodash/fp");

var _fp2 = _interopRequireDefault(_fp);

var _pdf_viewer = require("pdfjs-dist/web/pdf_viewer");

var _ToolBar = require("./ToolBar");

var _ToolBar2 = _interopRequireDefault(_ToolBar);

require("bootstrap/dist/css/bootstrap.min.css");

require("pdfjs-dist/web/pdf_viewer.css");

require("../style/pdf_viewer.css");

require("../style/PdfHighlighter.css");

var _getBoundingRect = require("../lib/get-bounding-rect");

var _getBoundingRect2 = _interopRequireDefault(_getBoundingRect);

var _getClientRects = require("../lib/get-client-rects");

var _getClientRects2 = _interopRequireDefault(_getClientRects);

var _getAreaAsPng = require("../lib/get-area-as-png");

var _getAreaAsPng2 = _interopRequireDefault(_getAreaAsPng);

var _pdfjsDom = require("../lib/pdfjs-dom");

var _TipContainer = require("./TipContainer");

var _TipContainer2 = _interopRequireDefault(_TipContainer);

var _MouseSelection = require("./MouseSelection");

var _MouseSelection2 = _interopRequireDefault(_MouseSelection);

var _coordinates = require("../lib/coordinates");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var EMPTY_ID = "empty-id";

var PdfHighlighter = function (_PureComponent) {
  _inherits(PdfHighlighter, _PureComponent);

  function PdfHighlighter() {
    var _temp, _this, _ret;

    _classCallCheck(this, PdfHighlighter);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, _PureComponent.call.apply(_PureComponent, [this].concat(args))), _this), _this.state = {
      ghostHighlight: null,
      isCollapsed: true,
      range: null,
      scrolledToHighlightId: EMPTY_ID,
      currentScale: 0.71,
      currentpage: 1
    }, _this.viewer = null, _this.containerNode = null, _this.onscrollPage = function (e) {
      if (_this.viewer && _this.state.currentpage !== _this.viewer.currentPageNumber) {
        _this.setState({ currentpage: _this.viewer.currentPageNumber });
      }
    }, _this.hideTipAndSelection = function () {
      var tipNode = (0, _pdfjsDom.findOrCreateContainerLayer)(_this.viewer.viewer, "PdfHighlighter__tip-layer");

      _reactDom2.default.unmountComponentAtNode(tipNode);

      _this.setState({ ghostHighlight: null, tip: null }, function () {
        return _this.renderHighlights();
      });
    }, _this.onTextLayerRendered = function () {
      _this.renderHighlights();
    }, _this.scrollTo = function (highlight) {
      var _highlight$position = highlight.position,
          pageNumber = _highlight$position.pageNumber,
          boundingRect = _highlight$position.boundingRect,
          usePdfCoordinates = _highlight$position.usePdfCoordinates;


      _this.viewer.container.removeEventListener("scroll", _this.onScroll);

      var pageViewport = _this.viewer.getPageView(pageNumber - 1).viewport;

      var scrollMargin = 10;

      _this.viewer.scrollPageIntoView({
        pageNumber: pageNumber,
        destArray: [null, { name: "XYZ" }].concat(pageViewport.convertToPdfPoint(0, (0, _coordinates.scaledToViewport)(boundingRect, pageViewport, usePdfCoordinates).top - scrollMargin), [0])
      });

      _this.setState({
        scrolledToHighlightId: highlight.id
      }, function () {
        return _this.renderHighlights();
      });

      // wait for scrolling to finish
      setTimeout(function () {
        _this.viewer.container.addEventListener("scroll", _this.onScroll);
      }, 100);
    }, _this.onDocumentReady = function () {
      var scrollRef = _this.props.scrollRef;


      _this.viewer.currentScaleValue = "auto";
      _this.setState({ currentScale: _this.viewer.currentScale });
      scrollRef(_this.scrollTo);
    }, _this.onSelectionChange = function () {
      var selection = window.getSelection();

      if (selection.isCollapsed) {
        _this.setState({ isCollapsed: true });
        return;
      }

      var range = selection.getRangeAt(0);

      if (!range) {
        return;
      }

      _this.setState({
        isCollapsed: false,
        range: range
      });

      _this.debouncedAfterSelection();
    }, _this.onScroll = function () {
      var onScrollChange = _this.props.onScrollChange;


      onScrollChange();

      _this.setState({
        scrolledToHighlightId: EMPTY_ID
      }, function () {
        return _this.renderHighlights();
      });

      _this.viewer.container.removeEventListener("scroll", _this.onScroll);
    }, _this.onMouseDown = function (event) {
      if (!(event.target instanceof HTMLElement)) {
        return;
      }

      if (event.target.closest(".PdfHighlighter__tip-container")) {
        return;
      }

      _this.hideTipAndSelection();
    }, _this.handleKeyDown = function (event) {
      if (event.code === "Escape") {
        _this.hideTipAndSelection();
      }
    }, _this.afterSelection = function () {
      var onSelectionFinished = _this.props.onSelectionFinished;
      var _this$state = _this.state,
          isCollapsed = _this$state.isCollapsed,
          range = _this$state.range;


      if (!range || isCollapsed) {
        return;
      }

      var page = (0, _pdfjsDom.getPageFromRange)(range);

      if (!page) {
        return;
      }

      var rects = (0, _getClientRects2.default)(range, page.node);

      if (rects.length === 0) {
        return;
      }

      var boundingRect = (0, _getBoundingRect2.default)(rects);

      var viewportPosition = { boundingRect: boundingRect, rects: rects, pageNumber: page.number };

      var content = {
        text: range.toString()
      };
      var scaledPosition = _this.viewportPositionToScaled(viewportPosition);

      _this.renderTipAtPosition(viewportPosition, onSelectionFinished(scaledPosition, content, function () {
        return _this.hideTipAndSelection();
      }, function () {
        return _this.setState({
          ghostHighlight: { position: scaledPosition }
        }, function () {
          return _this.renderHighlights();
        });
      }));
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  PdfHighlighter.prototype.componentDidUpdate = function componentDidUpdate(prevProps) {
    if (prevProps.highlights !== this.props.highlights) {
      this.renderHighlights(this.props);
    }
  };

  PdfHighlighter.prototype.componentDidMount = function componentDidMount() {
    var _this2 = this;

    var pdfDocument = this.props.pdfDocument;


    this.debouncedAfterSelection = _fp2.default.debounce(500, this.afterSelection);
    this.linkService = new _pdf_viewer.PDFLinkService();

    this.viewer = new _pdf_viewer.PDFViewer({
      container: this.containerNode,
      enhanceTextSelection: true,
      removePageBorders: true,
      linkService: this.linkService
    });

    this.viewer.setDocument(pdfDocument);
    this.linkService.setDocument(pdfDocument);
    this.linkService.setViewer(this.viewer);

    // debug
    window.PdfViewer = this;
    this.viewer.container.addEventListener('scroll', this.onscrollPage);
    document.addEventListener("selectionchange", this.onSelectionChange);
    document.addEventListener("keydown", this.handleKeyDown);

    this.containerNode && this.containerNode.addEventListener("pagesinit", function () {
      _this2.onDocumentReady();
    });

    this.containerNode && this.containerNode.addEventListener("textlayerrendered", this.onTextLayerRendered);
  };

  PdfHighlighter.prototype.componentWillUnmount = function componentWillUnmount() {
    this.viewer.container.removeEventListener('scroll', this.onscrollPage);
    document.removeEventListener("selectionchange", this.onSelectionChange);
    document.removeEventListener("keydown", this.handleKeyDown);

    this.containerNode && this.containerNode.removeEventListener("textlayerrendered", this.onTextLayerRendered);
  };

  PdfHighlighter.prototype.findOrCreateHighlightLayer = function findOrCreateHighlightLayer(page) {
    var textLayer = this.viewer.getPageView(page - 1).textLayer;

    if (!textLayer) {
      return null;
    }

    return (0, _pdfjsDom.findOrCreateContainerLayer)(textLayer.textLayerDiv, "PdfHighlighter__highlight-layer");
  };

  PdfHighlighter.prototype.groupHighlightsByPage = function groupHighlightsByPage(highlights) {
    var ghostHighlight = this.state.ghostHighlight;


    return [].concat(highlights, [ghostHighlight]).filter(Boolean).reduce(function (res, highlight) {
      var pageNumber = highlight.position.pageNumber;


      res[pageNumber] = res[pageNumber] || [];
      res[pageNumber].push(highlight);

      return res;
    }, {});
  };

  PdfHighlighter.prototype.showTip = function showTip(highlight, content) {
    var _state = this.state,
        isCollapsed = _state.isCollapsed,
        ghostHighlight = _state.ghostHighlight,
        isAreaSelectionInProgress = _state.isAreaSelectionInProgress;


    var highlightInProgress = !isCollapsed || ghostHighlight;

    if (highlightInProgress || isAreaSelectionInProgress) {
      return;
    }

    this.renderTipAtPosition(highlight.position, content);
  };

  PdfHighlighter.prototype.scaledPositionToViewport = function scaledPositionToViewport(_ref) {
    var pageNumber = _ref.pageNumber,
        boundingRect = _ref.boundingRect,
        rects = _ref.rects,
        usePdfCoordinates = _ref.usePdfCoordinates;

    var viewport = this.viewer.getPageView(pageNumber - 1).viewport;

    return {
      boundingRect: (0, _coordinates.scaledToViewport)(boundingRect, viewport, usePdfCoordinates),
      rects: (rects || []).map(function (rect) {
        return (0, _coordinates.scaledToViewport)(rect, viewport, usePdfCoordinates);
      }),
      pageNumber: pageNumber
    };
  };

  PdfHighlighter.prototype.viewportPositionToScaled = function viewportPositionToScaled(_ref2) {
    var pageNumber = _ref2.pageNumber,
        boundingRect = _ref2.boundingRect,
        rects = _ref2.rects;

    var viewport = this.viewer.getPageView(pageNumber - 1).viewport;

    return {
      boundingRect: (0, _coordinates.viewportToScaled)(boundingRect, viewport),
      rects: (rects || []).map(function (rect) {
        return (0, _coordinates.viewportToScaled)(rect, viewport);
      }),
      pageNumber: pageNumber
    };
  };

  PdfHighlighter.prototype.screenshot = function screenshot(position, pageNumber) {
    var canvas = this.viewer.getPageView(pageNumber - 1).canvas;

    return (0, _getAreaAsPng2.default)(canvas, position);
  };

  PdfHighlighter.prototype.renderHighlights = function renderHighlights(nextProps) {
    var _this3 = this;

    var _ref3 = nextProps || this.props,
        highlightTransform = _ref3.highlightTransform,
        highlights = _ref3.highlights;

    var pdfDocument = this.props.pdfDocument;
    var _state2 = this.state,
        tip = _state2.tip,
        scrolledToHighlightId = _state2.scrolledToHighlightId;


    var highlightsByPage = this.groupHighlightsByPage(highlights);

    var _loop = function _loop(_pageNumber) {
      var highlightLayer = _this3.findOrCreateHighlightLayer(_pageNumber);

      if (highlightLayer) {
        _reactDom2.default.render(_react2.default.createElement(
          "div",
          null,
          (highlightsByPage[String(_pageNumber)] || []).map(function (highlight, index) {
            var position = highlight.position,
                rest = _objectWithoutProperties(highlight, ["position"]);

            var viewportHighlight = _extends({
              position: _this3.scaledPositionToViewport(position)
            }, rest);

            if (tip && tip.highlight.id === String(highlight.id)) {
              _this3.showTip(tip.highlight, tip.callback(viewportHighlight));
            }

            var isScrolledTo = Boolean(scrolledToHighlightId === highlight.id);

            return highlightTransform(viewportHighlight, index, function (highlight, callback) {
              _this3.setState({
                tip: { highlight: highlight, callback: callback }
              });

              _this3.showTip(highlight, callback(highlight));
            }, _this3.hideTipAndSelection, function (rect) {
              var viewport = _this3.viewer.getPageView(_pageNumber - 1).viewport;

              return (0, _coordinates.viewportToScaled)(rect, viewport);
            }, function (boundingRect) {
              return _this3.screenshot(boundingRect, _pageNumber);
            }, isScrolledTo);
          })
        ), highlightLayer);
      }
    };

    for (var _pageNumber = 1; _pageNumber <= pdfDocument.numPages; _pageNumber++) {
      _loop(_pageNumber);
    }
  };

  PdfHighlighter.prototype.renderTipAtPosition = function renderTipAtPosition(position, inner) {
    var boundingRect = position.boundingRect,
        pageNumber = position.pageNumber;


    var page = {
      node: this.viewer.getPageView(pageNumber - 1).div
    };

    var pageBoundingRect = page.node.getBoundingClientRect();

    var tipNode = (0, _pdfjsDom.findOrCreateContainerLayer)(this.viewer.viewer, "PdfHighlighter__tip-layer");

    _reactDom2.default.render(_react2.default.createElement(_TipContainer2.default, {
      scrollTop: this.viewer.container.scrollTop,
      pageBoundingRect: pageBoundingRect,
      style: {
        left: page.node.offsetLeft + boundingRect.left + boundingRect.width / 2,
        top: boundingRect.top + page.node.offsetTop,
        bottom: boundingRect.top + page.node.offsetTop + boundingRect.height
      },
      children: inner
    }), tipNode);
  };

  PdfHighlighter.prototype.toggleTextSelection = function toggleTextSelection(flag) {
    this.viewer.viewer.classList.toggle("PdfHighlighter--disable-selection", flag);
  };

  PdfHighlighter.prototype.render = function render() {
    var _this4 = this;

    var _props = this.props,
        onSelectionFinished = _props.onSelectionFinished,
        enableAreaSelection = _props.enableAreaSelection;


    return _react2.default.createElement(
      _reactPointable2.default,
      { onPointerDown: this.onMouseDown },
      _react2.default.createElement(
        "div",
        {
          ref: function ref(node) {
            return _this4.containerNode = node;
          },
          className: "PdfHighlighter",
          onContextMenu: function onContextMenu(e) {
            return e.preventDefault();
          }
        },
        _react2.default.createElement("div", { className: "pdfViewer" }),
        typeof enableAreaSelection === "function" ? _react2.default.createElement(_MouseSelection2.default, {
          onDragStart: function onDragStart() {
            return _this4.toggleTextSelection(true);
          },
          onDragEnd: function onDragEnd() {
            return _this4.toggleTextSelection(false);
          },
          onChange: function onChange(isVisible) {
            return _this4.setState({ isAreaSelectionInProgress: isVisible });
          },
          shouldStart: function shouldStart(event) {
            return enableAreaSelection(event) && event.target instanceof HTMLElement && Boolean(event.target.closest(".page"));
          },
          onSelection: function onSelection(startTarget, boundingRect, resetSelection) {
            var page = (0, _pdfjsDom.getPageFromElement)(startTarget);

            if (!page) {
              return;
            }

            var pageBoundingRect = _extends({}, boundingRect, {
              top: boundingRect.top - page.node.offsetTop,
              left: boundingRect.left - page.node.offsetLeft
            });

            var viewportPosition = {
              boundingRect: pageBoundingRect,
              rects: [],
              pageNumber: page.number
            };

            var scaledPosition = _this4.viewportPositionToScaled(viewportPosition);

            var image = _this4.screenshot(pageBoundingRect, page.number);

            _this4.renderTipAtPosition(viewportPosition, onSelectionFinished(scaledPosition, { image: image }, function () {
              return _this4.hideTipAndSelection();
            }, function () {
              return _this4.setState({
                ghostHighlight: {
                  position: scaledPosition,
                  content: { image: image }
                }
              }, function () {
                resetSelection();
                _this4.renderHighlights();
              });
            }));
          }
        }) : null
      ),
      this.viewer ? _react2.default.createElement(_ToolBar2.default, {
        viewer: this.viewer,
        currentScale: this.state.currentScale,
        currentPageNum: this.state.currentpage
      }) : null
    );
  };

  return PdfHighlighter;
}(_react.PureComponent);

exports.default = PdfHighlighter;
module.exports = exports["default"];